'
' Template for user code replacement of compiler's I2C HBusIn/HBusOut library subroutines
' The exact mechanism used by the compiler is reproduced using BASIC commands
'
Reminders = Off
  'Optimiser_Level = 3                ' Uncomment for maximum optimisation
'
' Disable the compiler's HBusin and HBusout commands
'
  #disable HBusOut, hbusoutb, HBusIn, hbusinb, HBStart, HBReStart, HBusNack
'
' Declare the Pins used for hardware I2C communications
' These are fixed to the pins below, and cannot be changed
' Beacuse the commands are using the MSSP peripheral
'
  Declare SDA_Pin = PORTC.4           ' SDA connected to thisd pin
  Declare SCL_Pin = PORTC.3           ' SCL connected to this pin
'
' Create some compiler system variables for the routines
'
  Dim BPF As Byte System              ' Holds state flags
  Dim GPR As Byte System              ' General purpose
  Dim PP1 As Byte System              ' Used for loop counting
  Dim PP3 As Byte System              ' General purpose
  Dim PP4 As Byte System              ' Bit-0 returns the received Ack state
'
' I2C Flags
'
  Symbol tSkipAckAfterRead = BPF.2    ' I2C Skip Acknowledge after read If Set
  Symbol tReadWriteThenStop = BPF.4   ' I2C R/W Then Stop flag
  Symbol tSendStartBeforeRead = BPF.5 ' I2C reading flag. When Set, a Start, Command sequence is first initiated before a Byte is read
  Symbol tStartSent = BPF.6           ' I2C Start condition sent flag
  Symbol tBusyWriting = BPF.7         ' I2C writing flag

'-------------------------------------------------------------------------------------------------
  GoTo Over_HBusinHBusout_Subs        ' Jump over the subroutines
'-------------------------------------------------------------------------------------------------
' Hbusin
' Receive I2C data using MSSP
'
' Input     : None
' Output    : PP1, or Wreg hold the byte received
'           : Carry FLAG, or PP4.0 indicate ACK return
' Notes     : Do not disturb bits 0 or 1 of BPF as the compiler uses these elsewhere
'           : Call hbusinb to read a byte without sending an Acknowledge or Stop condition
'           : Call hbusin to read a byte and send an Acknowledge and Stop condition after it
'
#ifdef Hbusinb#Req
' Read a Byte from the bus without sending an Acknowledge or stop
@Hbusin#Req = 1                       ; Enable the subroutine
hbusinb:
  Clear tReadWriteThenStop            ' Indicate Don't send Stop condition
  Set tSendStartBeforeRead            ' Indicate No Start condition. And Command byte sent first
  Set tStartSent                      ' Indicate Not the first byte
  Set tBusyWriting                    ' Indicate No writing
  Set tSkipAckAfterRead               ' Indicate Don't Send acknowledges after writes
  GoTo Hbusin_ReceiveWithoutStop      ' Receive data byte without a Stop condition, or an Acknowledge sent
#endif

#ifdef Hbusin#Req
@Hbus_Wait_For_Idle#Req = 1           ; Enable the subroutine
@Hbusout#Req = 1                      ; Enable the subroutine
hbusin2:
  Set tReadWriteThenStop              ' Indicate Stop condition after this byte
  GoTo HBusinStopAfterByte            ' Skip
@hbusin
  Clear tReadWriteThenStop            ' Indicate Don't send Stop condition
HBusinStopAfterByte:
  If tSendStartBeforeRead = 0 Then    ' Already reading?
    Set tSendStartBeforeRead          ' No. So indicate read
    GoSub Hbusout_Restart             ' Send a Restart condition and command
    Bc Hbus_SendStop                  ' Acknowledged?
  EndIf
  Clear tSkipAckAfterRead             ' Send acknowledges after writes
  GoSub Hbus_WaitForIdle              ' Wait for the bus to become idle before loading SSPCON2 with the RCEN bit
Hbusin_ReceiveWithoutStop:
  Set SSPCON2bits_RCEN                ' Send Receive condition
  GoSub Hbus_WaitForIdle              ' Wait for the bus to become idle
  WREG = SSPBUF                       ' Place the buffer into WREG
  If tSkipAckAfterRead = 0 Then       ' Send Acknowledge condition? 0 = yes , 1 = no
    If tReadWriteThenStop = 1 Then    ' Send a stop condition?
      GoSub Hbus_SendStop             ' Yes. So send a Stop condition
      GoTo Hbusin_Exit                ' Exit the routine
    EndIf
    '
    ' Send acknowledge condition (Called when the HbusAck command is used)
    '
@hbus@ack
    Clear SSPCON2bits_ACKDT           ' Clear acknowledge bit for Ack
    Set SSPCON2bits_ACKEN             ' Send an Ack condition
    While SSPCON2bits_ACKEN = 1: Wend ' Wait until we are in idle mode
    If tReadWriteThenStop = 1 Then    ' Send a stop condition?
      GoSub Hbus_SendStop             ' Yes. So send a Stop condition
    EndIf
  EndIf
Hbusin_Exit:
  Clear STATUSbits_C                  ' Clear Carry for OK return
  PP1 = WREG                          ' Place result (held in WREG) into PP1 before returning
  GoTo Hbus_Exit
#endif

'-------------------------------------------------------------------------------------------------
' Hbusout
' Send I2C data using the MSSP module
'
' Input     : WREG = Data/Command
'           : GPR = command (used internally)
' Output    : Carry Flag, or PP4.0 indicate Ack return
' Notes     : BPF used as a state flag holder
'           : Do not disturb bits 0 or 1 of BPF as the compiler uses these elsewhere
'           : Call hbusoutb if a Stop condition is not required after a byte is sent
'           : Call hbusout2 if a Stop condition is required after a byte is sent
'
#ifdef Hbusoutb#Req
' Write a Byte to the bus without sending a Start and Command byte first
@Hbusout#Req = 1                        ; Enable the subroutine
hbusoutb:
  Set tSendStartBeforeRead              ' Indicate No Start condition. And Command byte sent first
  Set tStartSent                        ' Indicate Not the first byte
  Set tBusyWriting                      ' Indicate No writing
  GoTo Hbus_Out                         ' Send a byte without a Stop condition
#endif

#ifdef Hbusout#Req
@Hbus_Wait_For_Idle#Req = 1             ; Enable the subroutine
@HbStart#Req = 1                        ; Enable the subroutine
@HbRestart#Req = 1                      ; Enable the subroutine
hbusout2:
  Set tReadWriteThenStop                ' Indicate Stop condition after this byte
  GoTo HBusoutStopAfterByte             ' Skip
Hbus_Out:
@hbusout
  Clear tReadWriteThenStop              ' Indicate Don't send Stop condition
HBusoutStopAfterByte:
  If tStartSent = 0 Then                ' Check for first I2C byte (command)
    GPR = WREG & 254                    ' Yes. So make sure R/W bit clear
    Set tStartSent                      ' Indicate not first byte anymore
    Clear STATUSbits_C                  ' Set Carry for OK return
    GoTo Hbus_Exit                      ' Exit the routine
  EndIf
  If tBusyWriting = 0 Then              ' Already writing?
    PP3 = WREG                          ' No so save byte
    Set tBusyWriting                    ' Indicate write
    GoSub Hbusout_Start                 ' Send a Start condition and command
    If STATUSbits_C = 1 Then            ' Acknowledged?
      GoTo Hbus_SendStop                ' No. So send a Stop condition
    EndIf
    WREG = PP3                          ' Get byte back
  EndIf
Hbus_SSC:
  GoSub Hbus_WriteByte                  ' Write byte
  If STATUSbits_C = 0 Then              ' Stop if not Acknowledged?
    If tReadWriteThenStop = 0 Then      ' Send Stop condition?
      GoTo Hbus_Exit                    ' No. So Exit
    EndIf
  EndIf
'
' Send a Stop condition (Called when the HBStop command is used)
'
Hbus_SendStop:
@hbus@stp
  Set SSPCON2bits_PEN                   ' Send the Stop condition
  While SSPCON2bits_PEN = 1             ' Loop until a Stop condition has been sent?
    #ifdef WATCHDOG_REQ
    Clrwdt
    #endif
  Wend
  Clear tStartSent                      ' Indicate Reset first byte flag
  Clear tBusyWriting                    ' Indicate No longer writing
  Clear tSendStartBeforeRead            ' Indicate No longer reading either
  Clear tSkipAckAfterRead               ' Indicate Send acknowledges after reads
'
' Exit the I2C subroutines
' Setting or Clearing PP4.0 and Carry bit according to the Ack return
'
Hbus_Exit:
  PP4.0 = STATUSbits_C
  Return

Hbusout_Restart:
@hbus@rst
  GoSub Hbus_SendRestart              ' Send a Restart condition
  Bra $ + 4                           ' Skip over the Start condition call
Hbusout_Start:
@hbus@st
  GoSub Hbus_SendStart                ' Send a Start condition
  WREG = GPR                          ' Get command byte
  WREG.0 = tSendStartBeforeRead       ' Add the Read bit (if required)
'
' Send a byte to the bus
'
Hbus_WriteByte:
  SSPBUF = WREG                       ' Place the byte to output into the buffer
  Bra Hbus_WaitForIdleWithWrite       ' Wait for the Write to perform and the bus to become idle
#endif

#ifdef HBSTART#Req
'
' Send a Start condition (Called when the HBStart command is used)
'
Hbus_SendStart:
@hbus@strt
  Set __SCL_Port.__SCL_Pin            ' Set SCL pin to Input
  Set __SDA_Port.__SDA_Pin            ' Set SDA pin to Input
  Set SSPCON2bits_SEN                 ' Send a start condition
  While SSPCON2bits_SEN = 1           ' Wait until a Start condition has been sent
    #ifdef WATCHDOG_REQ
    Clrwdt
    #endif
  Wend
  Return                              ' Return from sending Start condition
#endif

#ifdef HBRESTART#Req
'
' Send a Restart condition (Called when the HBReStart command is used)
'
Hbus_SendRestart:
@hbus@restrt
  Set SSPCON2bits_RSEN                ' Send a Restart condition
  While SSPCON2bits_RSEN = 1          ' Wait until a Restart condition has been sent
    #ifdef WATCHDOG_REQ
    Clrwdt
    #endif
  Wend
  Return                              ' Return from sending Restart
#endif
'
' Wait for the I2C Bus to become idle before continuing
' A loop of 255 iterations will be performed if the bus is not ready
' If the loop completes, the Carry flag will be set to indicate that the I2C Bus was not ready
'
#ifdef Hbus_Wait_For_Idle#Req
Hbus_WaitForIdleWithWrite:
  While SSPSTATbits_R_W = 1           ' Wait for data transmit to complete
    #ifdef WATCHDOG_REQ
    Clrwdt
    #endif
  Wend
Hbus_WaitForIdle:
  PP1 = 255                           ' Use PP1 as a counting loop
  Repeat
    WREG = SSPCON2 & $1F              ' Get a copy of SSPCON2 and mask out the non-status bits
    DelayCS 4                         ' A delay of 4 cycles
    Clear STATUSbits_C                ' Default to Acknowledge received
    If STATUSbits_Z = 1 Then          ' Test for a zero state ?
      Return                          ' Zero so bus is ready, therefore exit with Carry clear
    EndIf
    Dec PP1                           ' Decrement the loop counter
  Until STATUSbits_Z = 1              ' Exit loop after 255 counts
  Set STATUSbits_C                    ' Set Carry to signal no Acknowledge received
  Return
#endif
'-------------------------------------------------------------------------------------------------
' Send a Nack to the I2C bus
#ifdef HBusNack#Req
@hbus@nack
  Set SSPCON2bits_ACKDT               ' Set acknowledge bit for Not Ack
  Set SSPCON2bits_ACKEN               ' Initiate bus acknowledge sequence
  While SSPCON2bits_ACKEN = 1
    #ifdef WATCHDOG_REQ
    Clrwdt
    #endif
  Wend
  Return
#endif
'-------------------------------------------------------------------------------------------------
Over_HBusinHBusout_Subs:


